package cz.novoj.convertor;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.PosixParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

/**
 * Dumb simple text file convertor and cleaner.
 *
 * @author Jan Novotn
 * @version $Id: $
 */
public class Convertor {
	private String sourceDirectory;
	private String targetDirectory;
	private String[] extensions;
	private String sourceEncoding;
	private String targetEncoding;

	public void setSourceDirectory(String sourceDirectory) {
		this.sourceDirectory = sourceDirectory;
	}

	public void setTargetDirectory(String targetDirectory) {
		this.targetDirectory = targetDirectory;
	}

	public void setExtensions(String[] extensions) {
		this.extensions = extensions;
	}

	public void setSourceEncoding(String sourceEncoding) {
		this.sourceEncoding = sourceEncoding;
	}

	public void setTargetEncoding(String targetEncoding) {
		this.targetEncoding = targetEncoding;
	}

	public String getSourceDirectory() {
		return sourceDirectory;
	}

	public String getTargetDirectory() {
		return targetDirectory;
	}

	public String[] getExtensions() {
		return extensions;
	}

	public String getSourceEncoding() {
		return sourceEncoding;
	}

	public String getTargetEncoding() {
		return targetEncoding;
	}

	public int convert() throws Exception {
		int result = 0;
		File sourceDirectoryFile = new File(sourceDirectory);
		int sourceDirectoryIndex = sourceDirectoryFile.getAbsolutePath().length();
		Iterator it = FileUtils.iterateFiles(
				new File(sourceDirectory),
				extensions,
				true
		);
		while(it.hasNext()) {
			File file = (File)it.next();
			String targetFilePath = file.getAbsolutePath().substring(sourceDirectoryIndex);
			File targetFile = new File(targetDirectory + targetFilePath);
			targetFile.getParentFile().mkdirs();
			if (!targetFile.exists()) {
				targetFile.createNewFile();
			}
			InputStream fis = new BufferedInputStream(new FileInputStream(file));
			OutputStream fos = null;
			try {
				String text = IOUtils.toString(fis, sourceEncoding);				
				fos = new BufferedOutputStream(new FileOutputStream(targetFile));
				IOUtils.write(text, fos, targetEncoding);
				System.out.println("Writing: " + targetFile.getAbsolutePath());
				result++;
			} finally {
				IOUtils.closeQuietly(fis);
				IOUtils.closeQuietly(fos);
			}
		}
		return result;
	}

	public int removeBOM() throws Exception {
		int result = 0;
		File sourceDirectoryFile = new File(sourceDirectory);
		int sourceDirectoryIndex = sourceDirectoryFile.getAbsolutePath().length();
		Iterator it = FileUtils.iterateFiles(
				new File(sourceDirectory),
				extensions,
				true
		);
		while(it.hasNext()) {
			File file = (File)it.next();
			String targetFilePath = file.getAbsolutePath().substring(sourceDirectoryIndex);
			File targetFile = new File(targetDirectory + targetFilePath);
			targetFile.getParentFile().mkdirs();
			if (!targetFile.exists()) {
				targetFile.createNewFile();
			}
			InputStream fis = new BufferedInputStream(new FileInputStream(file));
			OutputStream fos = null;
			try {
				byte[] bytes = IOUtils.toByteArray(fis);
				if (bytes[0] == -17 && bytes[1] == -69 && bytes[2] == -65) {
					byte[] updatedBytes = new byte[bytes.length-3];
					System.arraycopy(bytes, 3, updatedBytes, 0, bytes.length - 3);
					fos = new BufferedOutputStream(new FileOutputStream(targetFile));
					IOUtils.write(updatedBytes, fos);
					System.out.println("Writing: " + targetFile.getAbsolutePath());
					result++;
				}
			} finally {
				IOUtils.closeQuietly(fis);
				IOUtils.closeQuietly(fos);
			}
		}
		return result;
	}

	public static void main(String[] args) throws Exception {
		Options options = new Options();
		options.addOption("m", "mode", true, "Running mode (convert / removeBOM).");
		options.addOption("e", "extensions", true, "List of extensions delimited by comma.");
		options.addOption("s", "sourceDir", true, "Source directory");
		options.addOption("t", "targetDir", true, "Target directory");
		options.addOption("i", "sourceEncoding", true, "Source encoding");
		options.addOption("o", "targetEncoding", true, "Target encoding");
		CommandLineParser parser= new PosixParser();

		Convertor convertor = new Convertor();
		CommandLine cmd = parser.parse(options, args);

		if (!(cmd.hasOption("m") && cmd.hasOption("s") && cmd.hasOption("t"))) {
			HelpFormatter fmt = new HelpFormatter();
			fmt.printHelp("java -jar convert.jar", options);
			return;
		}

		String mode = cmd.getOptionValue("m");
		convertor.setSourceDirectory(cmd.getOptionValue("s"));
		convertor.setTargetDirectory(cmd.getOptionValue("t"));
		convertor.setSourceEncoding(cmd.getOptionValue("i", "windows-1250"));
		convertor.setTargetEncoding(cmd.getOptionValue("o", "utf-8"));

		List<String> extensionsList = new ArrayList<String>();
		String extensions = cmd.getOptionValue("e", "");
		for(StringTokenizer tk = new StringTokenizer(extensions, ",", false); tk.hasMoreTokens();) {
			String extension = tk.nextToken();
			extensionsList.add(extension);
		}
		convertor.setExtensions(extensionsList.toArray(new String[extensionsList.size()]));

		System.out.println("Conversion will read files from " + convertor.getSourceDirectory() + " of extensions: " + cmd.getOptionValue("e") + " and write them into " + convertor.getTargetDirectory());

		int result = 0;
		if ("convert".equalsIgnoreCase(mode)) {
			System.out.println("Starting encoding conversion (from " + convertor.getSourceEncoding() + " to " + convertor.getTargetEncoding() + ")");
			result = convertor.convert();
		} else if ("removeBOM".equalsIgnoreCase(mode)) {
			result = convertor.removeBOM();
		}

		System.out.println("Conversion done. " + result + " files modified.");
	}

}
